
# array of stoplisted words for the DBIx::TextIndex module
# all words must be in lower case

package DBIx::TextIndex::stop;

@words = qw(
a
about
after
all
also
an
and
any
are
as
at
be
because
been
but
by
can
co
corp
could
for
from
had
has
have
he
her
his
i
if
in
inc
into
is
it
its
last
like
many
me
miss
more
most
much
mr
mrs
ms
my
mz
no
not
of
on
one
only
or
other
out
over
s
says
she
should
so
some
such
than
that
the
their
them
there
these
they
this
those
thus
to
too
up
us
you
was
we
were
when
which
who
whose
will
with
would
);

1;
